/*
 * alxHTMLParser.h
 * 
 * @author Alexandros Ntoulas {ntoulas@cs.ucla.edu}
 * @version 1.0
 * @date 07/05/2003
 *
 * Header for the HTML parser
 */

/*
Copyright (c) 2003-2004, Alexandros Ntoulas {ntoulas@cs.ucla.edu}
University of California, Los Angeles
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

* Redistributions of source code must retain the above copyright 
notice, this list of conditions and the following disclaimer. 

* Redistributions in binary form must reproduce the above copyright 
notice, this list of conditions and the following disclaimer in the 
documentation and/or other materials provided with the distribution. 

* Neither the name of the University of California Los Angeles nor 
the names of its contributors may be used to endorse or promote 
products derived from this software without specific prior written 
permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __ALX_HTMLPARSER_H
#define __ALX_HTMLPARSER_H


#define ELEM_EOF -1

#define ELEM_TOKEN		2000
#define ELEM_PUNCT		2001
#define ELEM_UNMATCH		2003
#define ELEM_ENTITY		2004

#define ELEM_CLOSE		1000
#define ELEM_UNKNOWN		0
#define ELEM_UNKNOWN_C		(ELEM_UNKNOWN 	+ ELEM_CLOSE)

#define ELEM_A 			1
#define ELEM_A_C 		(ELEM_A		+ ELEM_CLOSE)
#define ELEM_ABBR 		2
#define ELEM_ABBR_C 		(ELEM_ABBR	+ ELEM_CLOSE)
#define ELEM_ACRONYM 		3
#define ELEM_ACRONYM_C 		(ELEM_ACRONYM	+ ELEM_CLOSE)
#define ELEM_ADDRESS 		4
#define ELEM_ADDRESS_C 		(ELEM_ADDRESS	+ ELEM_CLOSE)
#define ELEM_APPLET 		5
#define ELEM_APPLET_C 		(ELEM_APPLET	+ ELEM_CLOSE)
#define ELEM_AREA 		6
#define ELEM_AREA_C 		(ELEM_AREA	+ ELEM_CLOSE)
#define ELEM_B 			7
#define ELEM_B_C 		(ELEM_B		+ ELEM_CLOSE)
#define ELEM_BASE 		8
#define ELEM_BASE_C 		(ELEM_BASE	+ ELEM_CLOSE)
#define ELEM_BASEFONT 		9
#define ELEM_BASEFONT_C 	(ELEM_BASEFONT	+ ELEM_CLOSE)
#define ELEM_BDO 		10
#define ELEM_BDO_C 		(ELEM_BDO	+ ELEM_CLOSE)
#define ELEM_BIG 		11
#define ELEM_BIG_C 		(ELEM_BIG	+ ELEM_CLOSE)
#define ELEM_BLOCKQUOTE 	12
#define ELEM_BLOCKQUOTE_C 	(ELEM_BLOCKQUOTE+ ELEM_CLOSE)
#define ELEM_BODY 		13
#define ELEM_BODY_C 		(ELEM_BODY	+ ELEM_CLOSE)
#define ELEM_BR 		14
#define ELEM_BR_C 		(ELEM_BR	+ ELEM_CLOSE)
#define ELEM_BUTTON 		15
#define ELEM_BUTTON_C 		(ELEM_BUTTON	+ ELEM_CLOSE)
#define ELEM_CAPTION 		16
#define ELEM_CAPTION_C 		(ELEM_CAPTION	+ ELEM_CLOSE)
#define ELEM_CENTER 		17
#define ELEM_CENTER_C 		(ELEM_CENTER	+ ELEM_CLOSE)
#define ELEM_CITE 		18
#define ELEM_CITE_C 		(ELEM_CITE	+ ELEM_CLOSE)
#define ELEM_CODE 		19
#define ELEM_CODE_C 		(ELEM_CODE	+ ELEM_CLOSE)
#define ELEM_COL 		20
#define ELEM_COL_C 		(ELEM_COL	+ ELEM_CLOSE)
#define ELEM_COLGROUP 		21
#define ELEM_COLGROUP_C 	(ELEM_COLGROUP	+ ELEM_CLOSE)
#define ELEM_DD 		22
#define ELEM_DD_C 		(ELEM_DD	+ ELEM_CLOSE)
#define ELEM_DEL 		23
#define ELEM_DEL_C 		(ELEM_DEL	+ ELEM_CLOSE)
#define ELEM_DFN 		24
#define ELEM_DFN_C 		(ELEM_DFN	+ ELEM_CLOSE)
#define ELEM_DIR 		25
#define ELEM_DIR_C 		(ELEM_DIR	+ ELEM_CLOSE)
#define ELEM_DIV 		26
#define ELEM_DIV_C 		(ELEM_DIV	+ ELEM_CLOSE)
#define ELEM_DL 		27
#define ELEM_DL_C 		(ELEM_DL	+ ELEM_CLOSE)
#define ELEM_DT 		28
#define ELEM_DT_C 		(ELEM_DT	+ ELEM_CLOSE)
#define ELEM_EM 		29
#define ELEM_EM_C 		(ELEM_EM	+ ELEM_CLOSE)
#define ELEM_FIELDSET 		30
#define ELEM_FIELDSET_C 	(ELEM_FIELDSET	+ ELEM_CLOSE)
#define ELEM_FONT 		31
#define ELEM_FONT_C 		(ELEM_FONT	+ ELEM_CLOSE)
#define ELEM_FORM 		32
#define ELEM_FORM_C 		(ELEM_FORM	+ ELEM_CLOSE)
#define ELEM_FRAME 		33
#define ELEM_FRAME_C 		(ELEM_FRAME	+ ELEM_CLOSE)
#define ELEM_FRAMESET 		34
#define ELEM_FRAMESET_C 	(ELEM_FRAMESET	+ ELEM_CLOSE)
#define ELEM_H1 		35
#define ELEM_H1_C 		(ELEM_H1	+ ELEM_CLOSE)
#define ELEM_H2 		36
#define ELEM_H2_C 		(ELEM_H2	+ ELEM_CLOSE)
#define ELEM_H3 		37
#define ELEM_H3_C 		(ELEM_H3	+ ELEM_CLOSE)
#define ELEM_H4 		38
#define ELEM_H4_C 		(ELEM_H4	+ ELEM_CLOSE)
#define ELEM_H5 		39
#define ELEM_H5_C 		(ELEM_H5	+ ELEM_CLOSE)
#define ELEM_H6 		40
#define ELEM_H6_C 		(ELEM_H6	+ ELEM_CLOSE)
#define ELEM_HEAD 		41
#define ELEM_HEAD_C 		(ELEM_HEAD	+ ELEM_CLOSE)
#define ELEM_HR 		42
#define ELEM_HR_C 		(ELEM_HR	+ ELEM_CLOSE)
#define ELEM_HTML 		43
#define ELEM_HTML_C 		(ELEM_HTML	+ ELEM_CLOSE)
#define ELEM_I 			44
#define ELEM_I_C 		(ELEM_I		+ ELEM_CLOSE)
#define ELEM_IFRAME 		45
#define ELEM_IFRAME_C 		(ELEM_IFRAME	+ ELEM_CLOSE)
#define ELEM_IMG 		46
#define ELEM_IMG_C 		(ELEM_IMG	+ ELEM_CLOSE)
#define ELEM_INPUT 		47
#define ELEM_INPUT_C 		(ELEM_INPUT	+ ELEM_CLOSE)
#define ELEM_INS 		48
#define ELEM_INS_C 		(ELEM_INS	+ ELEM_CLOSE)
#define ELEM_ISINDEX 		49
#define ELEM_ISINDEX_C 		(ELEM_ISINDEX	+ ELEM_CLOSE)
#define ELEM_KBD 		50
#define ELEM_KBD_C 		(ELEM_KBD	+ ELEM_CLOSE)
#define ELEM_LABEL 		51
#define ELEM_LABEL_C 		(ELEM_LABEL	+ ELEM_CLOSE)
#define ELEM_LEGEND 		52
#define ELEM_LEGEND_C 		(ELEM_LEGEND	+ ELEM_CLOSE)
#define ELEM_LI 		53
#define ELEM_LI_C 		(ELEM_LI	+ ELEM_CLOSE)
#define ELEM_LINK 		54
#define ELEM_LINK_C 		(ELEM_LINK	+ ELEM_CLOSE)
#define ELEM_MAP 		55
#define ELEM_MAP_C 		(ELEM_MAP	+ ELEM_CLOSE)
#define ELEM_MENU 		56
#define ELEM_MENU_C 		(ELEM_MENU	+ ELEM_CLOSE)
#define ELEM_META 		57
#define ELEM_META_C 		(ELEM_META	+ ELEM_CLOSE)
#define ELEM_NOFRAMES 		58
#define ELEM_NOFRAMES_C 	(ELEM_NOFRAMES	+ ELEM_CLOSE)
#define ELEM_NOSCRIPT 		59
#define ELEM_NOSCRIPT_C 	(ELEM_NOSCRIPT	+ ELEM_CLOSE)
#define ELEM_OBJECT 		60
#define ELEM_OBJECT_C 		(ELEM_OBJECT	+ ELEM_CLOSE)
#define ELEM_OL 		61
#define ELEM_OL_C 		(ELEM_OL	+ ELEM_CLOSE)
#define ELEM_OPTGROUP 		62
#define ELEM_OPTGROUP_C 	(ELEM_OPTGROUP	+ ELEM_CLOSE)
#define ELEM_OPTION 		63
#define ELEM_OPTION_C 		(ELEM_OPTION	+ ELEM_CLOSE)
#define ELEM_P 			64
#define ELEM_P_C 		(ELEM_P		+ ELEM_CLOSE)
#define ELEM_PARAM 		65
#define ELEM_PARAM_C 		(ELEM_PARAM	+ ELEM_CLOSE)
#define ELEM_PRE 		66
#define ELEM_PRE_C 		(ELEM_PRE	+ ELEM_CLOSE)
#define ELEM_Q 			67
#define ELEM_Q_C 		(ELEM_Q		+ ELEM_CLOSE)
#define ELEM_S 			68
#define ELEM_S_C 		(ELEM_S		+ ELEM_CLOSE)
#define ELEM_SAMP 		69
#define ELEM_SAMP_C 		(ELEM_SAMP	+ ELEM_CLOSE)
#define ELEM_SCRIPT 		70
#define ELEM_SCRIPT_C 		(ELEM_SCRIPT	+ ELEM_CLOSE)
#define ELEM_SELECT 		71
#define ELEM_SELECT_C 		(ELEM_SELECT	+ ELEM_CLOSE)
#define ELEM_SMALL 		72
#define ELEM_SMALL_C 		(ELEM_SMALL	+ ELEM_CLOSE)
#define ELEM_SPAN 		73
#define ELEM_SPAN_C 		(ELEM_SPAN	+ ELEM_CLOSE)
#define ELEM_STRIKE 		74
#define ELEM_STRIKE_C 		(ELEM_STRIKE	+ ELEM_CLOSE)
#define ELEM_STRONG 		75
#define ELEM_STRONG_C 		(ELEM_STRONG	+ ELEM_CLOSE)
#define ELEM_STYLE 		76
#define ELEM_STYLE_C 		(ELEM_STYLE	+ ELEM_CLOSE)
#define ELEM_SUB 		77
#define ELEM_SUB_C 		(ELEM_SUB	+ ELEM_CLOSE)
#define ELEM_SUP 		78
#define ELEM_SUP_C 		(ELEM_SUP	+ ELEM_CLOSE)
#define ELEM_TABLE 		79
#define ELEM_TABLE_C 		(ELEM_TABLE	+ ELEM_CLOSE)
#define ELEM_TBODY 		80
#define ELEM_TBODY_C 		(ELEM_TBODY	+ ELEM_CLOSE)
#define ELEM_TD 		81
#define ELEM_TD_C 		(ELEM_TD	+ ELEM_CLOSE)
#define ELEM_TEXTAREA 		82
#define ELEM_TEXTAREA_C 	(ELEM_TEXTAREA	+ ELEM_CLOSE)
#define ELEM_TFOOT 		83
#define ELEM_TFOOT_C 		(ELEM_TFOOT	+ ELEM_CLOSE)
#define ELEM_TH 		84
#define ELEM_TH_C 		(ELEM_TH	+ ELEM_CLOSE)
#define ELEM_THEAD 		85
#define ELEM_THEAD_C 		(ELEM_THEAD	+ ELEM_CLOSE)
#define ELEM_TITLE 		86
#define ELEM_TITLE_C 		(ELEM_TITLE	+ ELEM_CLOSE)
#define ELEM_TR 		87
#define ELEM_TR_C 		(ELEM_TR	+ ELEM_CLOSE)
#define ELEM_TT 		88
#define ELEM_TT_C 		(ELEM_TT	+ ELEM_CLOSE)
#define ELEM_U 			89
#define ELEM_U_C 		(ELEM_U		+ ELEM_CLOSE)
#define ELEM_UL 		90
#define ELEM_UL_C 		(ELEM_UL	+ ELEM_CLOSE)
#define ELEM_VAR 		91
#define ELEM_VAR_C 		(ELEM_VAR	+ ELEM_CLOSE)

#define ELEM_COMMENT		92
#define ELEM_COMMENT_C		(ELEM_COMMENT	+ ELEM_CLOSE)

#define ELEM_COUNT      92

/* The parser uses pointers to functions, declared at the bottom */

/* Handles the closing of a tag */
int echoOpenTag(int id, char* tag, int len);

/* Handles the closing of a tag */
int echoCloseTag(int id, char* tag, int len);

/* Handles a found token */
int echoToken(int id, char* token, int len);

/* Handles a comment, in case you want for example
 * to strip out html but still show comments.  */
int echoComment(int id, char* comment, int len);

/* Handles punctuation marks */
int echoPunct(int id, char* punct, int len);

/* Handles unmatched chars */
int echoUnmatched(int id, char* str, int len);

/* Handles entities i.e. &stuff; */
int echoEntity(int ent);

/* uses the specified buffer */
void useString(char *str);
void useBuffer(char *str, int len);

/* deletes the current buffer */
void deleteCurrentBuf();

/* backtracks the parser */
void backtrack(int len);

/* a prototype for html_lex, so that we can start
 * the parser from the outside */
int html_lex();


/* These are pointers to functions describing parser actions. 
 * They are used for more flexibility, i.e. we may want to change an
 * action on open tag at some point during parsing or
 * do convenient things by declaring our own functions.
 * They are initially set to default values of printing out everything.
 * If you want to do different things, write your own functions
 * and set the respective pointers to your code.
 */

typedef int (*ACTION_PTR)(int, char*, int);
static ACTION_PTR _HANDLE_OPEN_TAG 	= &echoOpenTag;
static ACTION_PTR _HANDLE_CLOSE_TAG	= &echoCloseTag;
static ACTION_PTR _HANDLE_TOKEN    	= &echoToken;
static ACTION_PTR _HANDLE_COMMENT  	= &echoComment;
static ACTION_PTR _HANDLE_PUNCT    	= &echoPunct;
static ACTION_PTR _HANDLE_UNMATCHED	= &echoUnmatched;

typedef int (*ACTION_PTR_ENT)(int);
static ACTION_PTR_ENT _HANDLE_ENTITY   	= &echoEntity;

/* these are functions for changing the above handlers */
void setOpenTagHandler   (ACTION_PTR h);
void setCloseTagHandler  (ACTION_PTR h);
void setTokenHandler     (ACTION_PTR h);
void setCommentHandler   (ACTION_PTR h);
void setPunctHandler     (ACTION_PTR h);
void setUnmatchedHandler (ACTION_PTR h);
void setEntityHandler    (ACTION_PTR_ENT h);




#endif /* __ALX_HTMLPARSER_H */
