/*
 * alxHTMLParser.lex
 *
 * @author Alexandros Ntoulas {ntoulas@cs.ucla.edu}
 * @version 1.0
 * @date 07/05/2003
 * 
 * This file contains lex definitions and hookups for an
 * HTML parser and tokenizer.
 */

/*
Copyright (c) 2003-2004, Alexandros Ntoulas {ntoulas@cs.ucla.edu}
University of California, Los Angeles
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

* Redistributions of source code must retain the above copyright 
notice, this list of conditions and the following disclaimer. 

* Redistributions in binary form must reproduce the above copyright 
notice, this list of conditions and the following disclaimer in the 
documentation and/or other materials provided with the distribution. 

* Neither the name of the University of California Los Angeles nor 
the names of its contributors may be used to endorse or promote 
products derived from this software without specific prior written 
permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/




/*----------- definitions -----------*/
%{

#include "alxHTMLParser.h"



%}


%option outfile="alxHTMLParser.c"
%option prefix="html_"

   /*- TOKEN    is a regex for matching tokens, we should change
            this to identify tokens in other languages. A different
	    approach would be to consider as token the regex
	    [^ \t\n\r]+ (i.e non white spaces) but that would return
	    garbage if it was presented a binary file.
   TOKEN	[a-zA-Z0-9]
   */
TOKEN	[^ \n\r\t\<\>,\.\/\?:\";\'\{\}\[\]\|\\~`!@#\$%\^&\*\(\)\-\_\+\=]


A	(A|a)
ABBR	(A|a)(B|b)(B|b)(R|r)
ACRONYM	(A|a)(C|c)(R|r)(O|o)(N|n)(Y|y)(M|m)
ADDRESS	(A|a)(D|d)(D|d)(R|r)(E|e)(S|s)(S|s)
APPLET	(A|a)(P|p)(P|p)(L|l)(E|e)(T|t)
AREA	(A|a)(R|r)(E|e)(A|a)
B	(B|b)
BASE	(B|b)(A|a)(S|s)(E|e)
BASEFONT	(B|b)(A|a)(S|s)(E|e)(F|f)(O|o)(N|n)(T|t)
BDO	(B|b)(D|d)(O|o)
BIG	(B|b)(I|i)(G|g)
BLOCKQUOTE	(B|b)(L|l)(O|o)(C|c)(K|k)(Q|q)(U|u)(O|o)(T|t)(E|e)
BODY	(B|b)(O|o)(D|d)(Y|y)
BR	(B|b)(R|r)
BUTTON	(B|b)(U|u)(T|t)(T|t)(O|o)(N|n)
CAPTION	(C|c)(A|a)(P|p)(T|t)(I|i)(O|o)(N|n)
CENTER	(C|c)(E|e)(N|n)(T|t)(E|e)(R|r)
CITE	(C|c)(I|i)(T|t)(E|e)
CODE	(C|c)(O|o)(D|d)(E|e)
COL	(C|c)(O|o)(L|l)
COLGROUP	(C|c)(O|o)(L|l)(G|g)(R|r)(O|o)(U|u)(P|p)
DD	(D|d)(D|d)
DEL	(D|d)(E|e)(L|l)
DFN	(D|d)(F|f)(N|n)
DIR	(D|d)(I|i)(R|r)
DIV	(D|d)(I|i)(V|v)
DL	(D|d)(L|l)
DT	(D|d)(T|t)
EM	(E|e)(M|m)
FIELDSET	(F|f)(I|i)(E|e)(L|l)(D|d)(S|s)(E|e)(T|t)
FONT	(F|f)(O|o)(N|n)(T|t)
FORM	(F|f)(O|o)(R|r)(M|m)
FRAME	(F|f)(R|r)(A|a)(M|m)(E|e)
FRAMESET	(F|f)(R|r)(A|a)(M|m)(E|e)(S|s)(E|e)(T|t)
H1	(H|h)(1|1)
H2	(H|h)(2|2)
H3	(H|h)(3|3)
H4	(H|h)(4|4)
H5	(H|h)(5|5)
H6	(H|h)(6|6)
HEAD	(H|h)(E|e)(A|a)(D|d)
HR	(H|h)(R|r)
HTML	(H|h)(T|t)(M|m)(L|l)
I	(I|i)
IFRAME	(I|i)(F|f)(R|r)(A|a)(M|m)(E|e)
IMG	(I|i)(M|m)(G|g)
INPUT	(I|i)(N|n)(P|p)(U|u)(T|t)
INS	(I|i)(N|n)(S|s)
ISINDEX	(I|i)(S|s)(I|i)(N|n)(D|d)(E|e)(X|x)
KBD	(K|k)(B|b)(D|d)
LABEL	(L|l)(A|a)(B|b)(E|e)(L|l)
LEGEND	(L|l)(E|e)(G|g)(E|e)(N|n)(D|d)
LI	(L|l)(I|i)
LINK	(L|l)(I|i)(N|n)(K|k)
MAP	(M|m)(A|a)(P|p)
MENU	(M|m)(E|e)(N|n)(U|u)
META	(M|m)(E|e)(T|t)(A|a)
NOFRAMES	(N|n)(O|o)(F|f)(R|r)(A|a)(M|m)(E|e)(S|s)
NOSCRIPT	(N|n)(O|o)(S|s)(C|c)(R|r)(I|i)(P|p)(T|t)
OBJECT	(O|o)(B|b)(J|j)(E|e)(C|c)(T|t)
OL	(O|o)(L|l)
OPTGROUP	(O|o)(P|p)(T|t)(G|g)(R|r)(O|o)(U|u)(P|p)
OPTION	(O|o)(P|p)(T|t)(I|i)(O|o)(N|n)
P	(P|p)
PARAM	(P|p)(A|a)(R|r)(A|a)(M|m)
PRE	(P|p)(R|r)(E|e)
Q	(Q|q)
S	(S|s)
SAMP	(S|s)(A|a)(M|m)(P|p)
SCRIPT	(S|s)(C|c)(R|r)(I|i)(P|p)(T|t)
SELECT	(S|s)(E|e)(L|l)(E|e)(C|c)(T|t)
SMALL	(S|s)(M|m)(A|a)(L|l)(L|l)
SPAN	(S|s)(P|p)(A|a)(N|n)
STRIKE	(S|s)(T|t)(R|r)(I|i)(K|k)(E|e)
STRONG	(S|s)(T|t)(R|r)(O|o)(N|n)(G|g)
STYLE	(S|s)(T|t)(Y|y)(L|l)(E|e)
SUB	(S|s)(U|u)(B|b)
SUP	(S|s)(U|u)(P|p)
TABLE	(T|t)(A|a)(B|b)(L|l)(E|e)
TBODY	(T|t)(B|b)(O|o)(D|d)(Y|y)
TD	(T|t)(D|d)
TEXTAREA	(T|t)(E|e)(X|x)(T|t)(A|a)(R|r)(E|e)(A|a)
TFOOT	(T|t)(F|f)(O|o)(O|o)(T|t)
TH	(T|t)(H|h)
THEAD	(T|t)(H|h)(E|e)(A|a)(D|d)
TITLE	(T|t)(I|i)(T|t)(L|l)(E|e)
TR	(T|t)(R|r)
TT	(T|t)(T|t)
U	(U|u)
UL	(U|u)(L|l)
VAR	(V|v)(A|a)(R|r)



  /*----------- rules -----------*/

%%
	/* comments */
\<![ \r\n\t]*(--([^\-]|[\r\n]|-[^\-])*--[ \r\n\t]*)\>	{if (_HANDLE_COMMENT)  _HANDLE_COMMENT  (ELEM_COMMENT, yytext, yyleng); return ELEM_COMMENT; }
\<![^>]*\>	{if (_HANDLE_COMMENT)  _HANDLE_COMMENT  (ELEM_COMMENT, yytext, yyleng); return ELEM_COMMENT; }

	/*
         * HTML elements
         * the order that the rules appear here is important 
	 */

\<\/{VAR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_VAR_C, yytext, yyleng); 
	return ELEM_VAR_C; 	}
\<{VAR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_VAR, yytext, yyleng); 
	return ELEM_VAR; 	}
	
\<\/{UL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_UL_C, yytext, yyleng); 
	return ELEM_UL_C; 	}
\<{UL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_UL, yytext, yyleng); 
	return ELEM_UL; 	}
	
\<\/{U}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_U_C, yytext, yyleng); 
	return ELEM_U_C; 	}
\<{U}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_U, yytext, yyleng); 
	return ELEM_U; 	}
	
\<\/{TT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TT_C, yytext, yyleng); 
	return ELEM_TT_C; 	}
\<{TT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TT, yytext, yyleng); 
	return ELEM_TT; 	}
	
\<\/{TR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TR_C, yytext, yyleng); 
	return ELEM_TR_C; 	}
\<{TR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TR, yytext, yyleng); 
	return ELEM_TR; 	}
	
\<\/{TITLE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TITLE_C, yytext, yyleng); 
	return ELEM_TITLE_C; 	}
\<{TITLE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TITLE, yytext, yyleng); 
	return ELEM_TITLE; 	}
	
\<\/{THEAD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_THEAD_C, yytext, yyleng); 
	return ELEM_THEAD_C; 	}
\<{THEAD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_THEAD, yytext, yyleng); 
	return ELEM_THEAD; 	}
	
\<\/{TH}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TH_C, yytext, yyleng); 
	return ELEM_TH_C; 	}
\<{TH}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TH, yytext, yyleng); 
	return ELEM_TH; 	}
	
\<\/{TFOOT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TFOOT_C, yytext, yyleng); 
	return ELEM_TFOOT_C; 	}
\<{TFOOT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TFOOT, yytext, yyleng); 
	return ELEM_TFOOT; 	}
	
\<\/{TEXTAREA}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TEXTAREA_C, yytext, yyleng); 
	return ELEM_TEXTAREA_C; 	}
\<{TEXTAREA}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TEXTAREA, yytext, yyleng); 
	return ELEM_TEXTAREA; 	}
	
\<\/{TD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TD_C, yytext, yyleng); 
	return ELEM_TD_C; 	}
\<{TD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TD, yytext, yyleng); 
	return ELEM_TD; 	}
	
\<\/{TBODY}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TBODY_C, yytext, yyleng); 
	return ELEM_TBODY_C; 	}
\<{TBODY}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TBODY, yytext, yyleng); 
	return ELEM_TBODY; 	}
	
\<\/{TABLE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_TABLE_C, yytext, yyleng); 
	return ELEM_TABLE_C; 	}
\<{TABLE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_TABLE, yytext, yyleng); 
	return ELEM_TABLE; 	}
	
\<\/{SUP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_SUP_C, yytext, yyleng); 
	return ELEM_SUP_C; 	}
\<{SUP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_SUP, yytext, yyleng); 
	return ELEM_SUP; 	}
	
\<\/{SUB}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_SUB_C, yytext, yyleng); 
	return ELEM_SUB_C; 	}
\<{SUB}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_SUB, yytext, yyleng); 
	return ELEM_SUB; 	}
	
\<\/{STYLE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_STYLE_C, yytext, yyleng); 
	return ELEM_STYLE_C; 	}
\<{STYLE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_STYLE, yytext, yyleng); 
	return ELEM_STYLE; 	}
	
\<\/{STRONG}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_STRONG_C, yytext, yyleng); 
	return ELEM_STRONG_C; 	}
\<{STRONG}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_STRONG, yytext, yyleng); 
	return ELEM_STRONG; 	}
	
\<\/{STRIKE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_STRIKE_C, yytext, yyleng); 
	return ELEM_STRIKE_C; 	}
\<{STRIKE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_STRIKE, yytext, yyleng); 
	return ELEM_STRIKE; 	}
	
\<\/{SPAN}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_SPAN_C, yytext, yyleng); 
	return ELEM_SPAN_C; 	}
\<{SPAN}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_SPAN, yytext, yyleng); 
	return ELEM_SPAN; 	}
	
\<\/{SMALL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_SMALL_C, yytext, yyleng); 
	return ELEM_SMALL_C; 	}
\<{SMALL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_SMALL, yytext, yyleng); 
	return ELEM_SMALL; 	}
	
\<\/{SELECT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_SELECT_C, yytext, yyleng); 
	return ELEM_SELECT_C; 	}
\<{SELECT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_SELECT, yytext, yyleng); 
	return ELEM_SELECT; 	}
	
\<\/{SCRIPT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_SCRIPT_C, yytext, yyleng); 
	return ELEM_SCRIPT_C; 	}
\<{SCRIPT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_SCRIPT, yytext, yyleng); 
	return ELEM_SCRIPT; 	}
	
\<\/{SAMP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_SAMP_C, yytext, yyleng); 
	return ELEM_SAMP_C; 	}
\<{SAMP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_SAMP, yytext, yyleng); 
	return ELEM_SAMP; 	}
	
\<\/{S}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_S_C, yytext, yyleng); 
	return ELEM_S_C; 	}
\<{S}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_S, yytext, yyleng); 
	return ELEM_S; 	}
	
\<\/{Q}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_Q_C, yytext, yyleng); 
	return ELEM_Q_C; 	}
\<{Q}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_Q, yytext, yyleng); 
	return ELEM_Q; 	}
	
\<\/{PRE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_PRE_C, yytext, yyleng); 
	return ELEM_PRE_C; 	}
\<{PRE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_PRE, yytext, yyleng); 
	return ELEM_PRE; 	}
	
\<\/{PARAM}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_PARAM_C, yytext, yyleng); 
	return ELEM_PARAM_C; 	}
\<{PARAM}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_PARAM, yytext, yyleng); 
	return ELEM_PARAM; 	}
	
\<\/{P}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_P_C, yytext, yyleng); 
	return ELEM_P_C; 	}
\<{P}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_P, yytext, yyleng); 
	return ELEM_P; 	}
	
\<\/{OPTION}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_OPTION_C, yytext, yyleng); 
	return ELEM_OPTION_C; 	}
\<{OPTION}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_OPTION, yytext, yyleng); 
	return ELEM_OPTION; 	}
	
\<\/{OPTGROUP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_OPTGROUP_C, yytext, yyleng); 
	return ELEM_OPTGROUP_C; 	}
\<{OPTGROUP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_OPTGROUP, yytext, yyleng); 
	return ELEM_OPTGROUP; 	}
	
\<\/{OL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_OL_C, yytext, yyleng); 
	return ELEM_OL_C; 	}
\<{OL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_OL, yytext, yyleng); 
	return ELEM_OL; 	}
	
\<\/{OBJECT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_OBJECT_C, yytext, yyleng); 
	return ELEM_OBJECT_C; 	}
\<{OBJECT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_OBJECT, yytext, yyleng); 
	return ELEM_OBJECT; 	}
	
\<\/{NOSCRIPT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_NOSCRIPT_C, yytext, yyleng); 
	return ELEM_NOSCRIPT_C; 	}
\<{NOSCRIPT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_NOSCRIPT, yytext, yyleng); 
	return ELEM_NOSCRIPT; 	}
	
\<\/{NOFRAMES}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_NOFRAMES_C, yytext, yyleng); 
	return ELEM_NOFRAMES_C; 	}
\<{NOFRAMES}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_NOFRAMES, yytext, yyleng); 
	return ELEM_NOFRAMES; 	}
	
\<\/{META}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_META_C, yytext, yyleng); 
	return ELEM_META_C; 	}
\<{META}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_META, yytext, yyleng); 
	return ELEM_META; 	}
	
\<\/{MENU}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_MENU_C, yytext, yyleng); 
	return ELEM_MENU_C; 	}
\<{MENU}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_MENU, yytext, yyleng); 
	return ELEM_MENU; 	}
	
\<\/{MAP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_MAP_C, yytext, yyleng); 
	return ELEM_MAP_C; 	}
\<{MAP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_MAP, yytext, yyleng); 
	return ELEM_MAP; 	}
	
\<\/{LINK}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_LINK_C, yytext, yyleng); 
	return ELEM_LINK_C; 	}
\<{LINK}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_LINK, yytext, yyleng); 
	return ELEM_LINK; 	}
	
\<\/{LI}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_LI_C, yytext, yyleng); 
	return ELEM_LI_C; 	}
\<{LI}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_LI, yytext, yyleng); 
	return ELEM_LI; 	}
	
\<\/{LEGEND}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_LEGEND_C, yytext, yyleng); 
	return ELEM_LEGEND_C; 	}
\<{LEGEND}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_LEGEND, yytext, yyleng); 
	return ELEM_LEGEND; 	}
	
\<\/{LABEL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_LABEL_C, yytext, yyleng); 
	return ELEM_LABEL_C; 	}
\<{LABEL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_LABEL, yytext, yyleng); 
	return ELEM_LABEL; 	}
	
\<\/{KBD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_KBD_C, yytext, yyleng); 
	return ELEM_KBD_C; 	}
\<{KBD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_KBD, yytext, yyleng); 
	return ELEM_KBD; 	}
	
\<\/{ISINDEX}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_ISINDEX_C, yytext, yyleng); 
	return ELEM_ISINDEX_C; 	}
\<{ISINDEX}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_ISINDEX, yytext, yyleng); 
	return ELEM_ISINDEX; 	}
	
\<\/{INS}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_INS_C, yytext, yyleng); 
	return ELEM_INS_C; 	}
\<{INS}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_INS, yytext, yyleng); 
	return ELEM_INS; 	}
	
\<\/{INPUT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_INPUT_C, yytext, yyleng); 
	return ELEM_INPUT_C; 	}
\<{INPUT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_INPUT, yytext, yyleng); 
	return ELEM_INPUT; 	}
	
\<\/{IMG}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_IMG_C, yytext, yyleng); 
	return ELEM_IMG_C; 	}
\<{IMG}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_IMG, yytext, yyleng); 
	return ELEM_IMG; 	}
	
\<\/{IFRAME}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_IFRAME_C, yytext, yyleng); 
	return ELEM_IFRAME_C; 	}
\<{IFRAME}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_IFRAME, yytext, yyleng); 
	return ELEM_IFRAME; 	}
	
\<\/{I}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_I_C, yytext, yyleng); 
	return ELEM_I_C; 	}
\<{I}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_I, yytext, yyleng); 
	return ELEM_I; 	}
	
\<\/{HTML}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_HTML_C, yytext, yyleng); 
	return ELEM_HTML_C; 	}
\<{HTML}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_HTML, yytext, yyleng); 
	return ELEM_HTML; 	}
	
\<\/{HR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_HR_C, yytext, yyleng); 
	return ELEM_HR_C; 	}
\<{HR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_HR, yytext, yyleng); 
	return ELEM_HR; 	}
	
\<\/{HEAD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_HEAD_C, yytext, yyleng); 
	return ELEM_HEAD_C; 	}
\<{HEAD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_HEAD, yytext, yyleng); 
	return ELEM_HEAD; 	}
	
\<\/{H6}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_H6_C, yytext, yyleng); 
	return ELEM_H6_C; 	}
\<{H6}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_H6, yytext, yyleng); 
	return ELEM_H6; 	}
	
\<\/{H5}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_H5_C, yytext, yyleng); 
	return ELEM_H5_C; 	}
\<{H5}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_H5, yytext, yyleng); 
	return ELEM_H5; 	}
	
\<\/{H4}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_H4_C, yytext, yyleng); 
	return ELEM_H4_C; 	}
\<{H4}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_H4, yytext, yyleng); 
	return ELEM_H4; 	}
	
\<\/{H3}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_H3_C, yytext, yyleng); 
	return ELEM_H3_C; 	}
\<{H3}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_H3, yytext, yyleng); 
	return ELEM_H3; 	}
	
\<\/{H2}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_H2_C, yytext, yyleng); 
	return ELEM_H2_C; 	}
\<{H2}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_H2, yytext, yyleng); 
	return ELEM_H2; 	}
	
\<\/{H1}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_H1_C, yytext, yyleng); 
	return ELEM_H1_C; 	}
\<{H1}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_H1, yytext, yyleng); 
	return ELEM_H1; 	}
	
\<\/{FRAMESET}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_FRAMESET_C, yytext, yyleng); 
	return ELEM_FRAMESET_C; 	}
\<{FRAMESET}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_FRAMESET, yytext, yyleng); 
	return ELEM_FRAMESET; 	}
	
\<\/{FRAME}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_FRAME_C, yytext, yyleng); 
	return ELEM_FRAME_C; 	}
\<{FRAME}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_FRAME, yytext, yyleng); 
	return ELEM_FRAME; 	}
	
\<\/{FORM}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_FORM_C, yytext, yyleng); 
	return ELEM_FORM_C; 	}
\<{FORM}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_FORM, yytext, yyleng); 
	return ELEM_FORM; 	}
	
\<\/{FONT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_FONT_C, yytext, yyleng); 
	return ELEM_FONT_C; 	}
\<{FONT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_FONT, yytext, yyleng); 
	return ELEM_FONT; 	}
	
\<\/{FIELDSET}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_FIELDSET_C, yytext, yyleng); 
	return ELEM_FIELDSET_C; 	}
\<{FIELDSET}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_FIELDSET, yytext, yyleng); 
	return ELEM_FIELDSET; 	}
	
\<\/{EM}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_EM_C, yytext, yyleng); 
	return ELEM_EM_C; 	}
\<{EM}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_EM, yytext, yyleng); 
	return ELEM_EM; 	}
	
\<\/{DT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_DT_C, yytext, yyleng); 
	return ELEM_DT_C; 	}
\<{DT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_DT, yytext, yyleng); 
	return ELEM_DT; 	}
	
\<\/{DL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_DL_C, yytext, yyleng); 
	return ELEM_DL_C; 	}
\<{DL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_DL, yytext, yyleng); 
	return ELEM_DL; 	}
	
\<\/{DIV}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_DIV_C, yytext, yyleng); 
	return ELEM_DIV_C; 	}
\<{DIV}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_DIV, yytext, yyleng); 
	return ELEM_DIV; 	}
	
\<\/{DIR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_DIR_C, yytext, yyleng); 
	return ELEM_DIR_C; 	}
\<{DIR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_DIR, yytext, yyleng); 
	return ELEM_DIR; 	}
	
\<\/{DFN}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_DFN_C, yytext, yyleng); 
	return ELEM_DFN_C; 	}
\<{DFN}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_DFN, yytext, yyleng); 
	return ELEM_DFN; 	}
	
\<\/{DEL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_DEL_C, yytext, yyleng); 
	return ELEM_DEL_C; 	}
\<{DEL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_DEL, yytext, yyleng); 
	return ELEM_DEL; 	}
	
\<\/{DD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_DD_C, yytext, yyleng); 
	return ELEM_DD_C; 	}
\<{DD}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_DD, yytext, yyleng); 
	return ELEM_DD; 	}
	
\<\/{COLGROUP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_COLGROUP_C, yytext, yyleng); 
	return ELEM_COLGROUP_C; 	}
\<{COLGROUP}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_COLGROUP, yytext, yyleng); 
	return ELEM_COLGROUP; 	}
	
\<\/{COL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_COL_C, yytext, yyleng); 
	return ELEM_COL_C; 	}
\<{COL}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_COL, yytext, yyleng); 
	return ELEM_COL; 	}
	
\<\/{CODE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_CODE_C, yytext, yyleng); 
	return ELEM_CODE_C; 	}
\<{CODE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_CODE, yytext, yyleng); 
	return ELEM_CODE; 	}
	
\<\/{CITE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_CITE_C, yytext, yyleng); 
	return ELEM_CITE_C; 	}
\<{CITE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_CITE, yytext, yyleng); 
	return ELEM_CITE; 	}
	
\<\/{CENTER}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_CENTER_C, yytext, yyleng); 
	return ELEM_CENTER_C; 	}
\<{CENTER}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_CENTER, yytext, yyleng); 
	return ELEM_CENTER; 	}
	
\<\/{CAPTION}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_CAPTION_C, yytext, yyleng); 
	return ELEM_CAPTION_C; 	}
\<{CAPTION}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_CAPTION, yytext, yyleng); 
	return ELEM_CAPTION; 	}
	
\<\/{BUTTON}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_BUTTON_C, yytext, yyleng); 
	return ELEM_BUTTON_C; 	}
\<{BUTTON}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_BUTTON, yytext, yyleng); 
	return ELEM_BUTTON; 	}
	
\<\/{BR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_BR_C, yytext, yyleng); 
	return ELEM_BR_C; 	}
\<{BR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_BR, yytext, yyleng); 
	return ELEM_BR; 	}
	
\<\/{BODY}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_BODY_C, yytext, yyleng); 
	return ELEM_BODY_C; 	}
\<{BODY}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_BODY, yytext, yyleng); 
	return ELEM_BODY; 	}
	
\<\/{BLOCKQUOTE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_BLOCKQUOTE_C, yytext, yyleng); 
	return ELEM_BLOCKQUOTE_C; 	}
\<{BLOCKQUOTE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_BLOCKQUOTE, yytext, yyleng); 
	return ELEM_BLOCKQUOTE; 	}
	
\<\/{BIG}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_BIG_C, yytext, yyleng); 
	return ELEM_BIG_C; 	}
\<{BIG}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_BIG, yytext, yyleng); 
	return ELEM_BIG; 	}
	
\<\/{BDO}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_BDO_C, yytext, yyleng); 
	return ELEM_BDO_C; 	}
\<{BDO}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_BDO, yytext, yyleng); 
	return ELEM_BDO; 	}
	
\<\/{BASEFONT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_BASEFONT_C, yytext, yyleng); 
	return ELEM_BASEFONT_C; 	}
\<{BASEFONT}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_BASEFONT, yytext, yyleng); 
	return ELEM_BASEFONT; 	}
	
\<\/{BASE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_BASE_C, yytext, yyleng); 
	return ELEM_BASE_C; 	}
\<{BASE}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_BASE, yytext, yyleng); 
	return ELEM_BASE; 	}
	
\<\/{B}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_B_C, yytext, yyleng); 
	return ELEM_B_C; 	}
\<{B}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_B, yytext, yyleng); 
	return ELEM_B; 	}
	
\<\/{AREA}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_AREA_C, yytext, yyleng); 
	return ELEM_AREA_C; 	}
\<{AREA}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_AREA, yytext, yyleng); 
	return ELEM_AREA; 	}
	
\<\/{APPLET}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_APPLET_C, yytext, yyleng); 
	return ELEM_APPLET_C; 	}
\<{APPLET}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_APPLET, yytext, yyleng); 
	return ELEM_APPLET; 	}
	
\<\/{ADDRESS}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_ADDRESS_C, yytext, yyleng); 
	return ELEM_ADDRESS_C; 	}
\<{ADDRESS}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_ADDRESS, yytext, yyleng); 
	return ELEM_ADDRESS; 	}
	
\<\/{ACRONYM}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_ACRONYM_C, yytext, yyleng); 
	return ELEM_ACRONYM_C; 	}
\<{ACRONYM}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_ACRONYM, yytext, yyleng); 
	return ELEM_ACRONYM; 	}
	
\<\/{ABBR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_ABBR_C, yytext, yyleng); 
	return ELEM_ABBR_C; 	}
\<{ABBR}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_ABBR, yytext, yyleng); 
	return ELEM_ABBR; 	}
	
\<\/{A}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_CLOSE_TAG)	_HANDLE_CLOSE_TAG (ELEM_A_C, yytext, yyleng); 
	return ELEM_A_C; 	}

\<{A}([^>'"]|\"[^"]*\"|'[^']*')*\>	{
	if (_HANDLE_OPEN_TAG)	_HANDLE_OPEN_TAG  (ELEM_A, yytext, yyleng); 
	return ELEM_A; 	}

	/* generic rules */
\<\/([^>'"!]|\"[^"]*\"|'[^']*')+\>	{
                 if (_HANDLE_CLOSE_TAG)_HANDLE_CLOSE_TAG(ELEM_UNKNOWN_C, yytext, yyleng); return ELEM_UNKNOWN_C; }
\<([^>'"!]|\"[^"]*\"|'[^']*')+\>	{
                 if (_HANDLE_OPEN_TAG) _HANDLE_OPEN_TAG (ELEM_UNKNOWN, yytext, yyleng); return ELEM_UNKNOWN; }

 	/* the following are replacements for valid entities of HTML */
&#[0-9]+;?      {if (_HANDLE_ENTITY)   _HANDLE_ENTITY (atoi(yytext+2)); return ELEM_ENTITY; }

&lt;?           {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      ('<'); return ELEM_ENTITY; }
&gt;?           {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      ('>'); return ELEM_ENTITY; }
&amp;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      ('&'); return ELEM_ENTITY; }
&quot;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      ('"'); return ELEM_ENTITY; }

&nbsp;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (' '); return ELEM_ENTITY; } // we could do putchar(160);
&iexcl;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (161); return ELEM_ENTITY; } // inverted exclamation mark
&cent;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (162); return ELEM_ENTITY; } // cent sign
&pound;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (163); return ELEM_ENTITY; } // pound sterling sign CURRENCY
&curren;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (164); return ELEM_ENTITY; } // general currency sign
&yen;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (165); return ELEM_ENTITY; } // yen sign
&brvbar;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (166); return ELEM_ENTITY; } // broken (vertical) bar
&sect;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (167); return ELEM_ENTITY; } // section sign
&uml;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (168); return ELEM_ENTITY; } // umlaut (dieresis)
&copy;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (169); return ELEM_ENTITY; } // copyright sign
&ordf;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (170); return ELEM_ENTITY; } // ordinal indicator, feminine
&laquo;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (171); return ELEM_ENTITY; } // angle quotation mark, left
&not;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (172); return ELEM_ENTITY; } // not sign
&shy;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (173); return ELEM_ENTITY; } // soft hyphen
&reg;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (174); return ELEM_ENTITY; } // registered sign
&macr;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (175); return ELEM_ENTITY; } // macron
&deg;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (176); return ELEM_ENTITY; } // degree sign
&plusmn;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (177); return ELEM_ENTITY; } // plus-or-minus sign
&sup2;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (178); return ELEM_ENTITY; } // superscript two
&sup3;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (179); return ELEM_ENTITY; } // superscript three
&acute;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (180); return ELEM_ENTITY; } // acute accent
&micro;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (181); return ELEM_ENTITY; } // micro sign
&para;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (182); return ELEM_ENTITY; } // pilcrow (paragraph sign)
&middot;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (183); return ELEM_ENTITY; } // middle dot
&cedil;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (184); return ELEM_ENTITY; } // cedilla
&sup1;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (185); return ELEM_ENTITY; } // superscript one
&ordm;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (186); return ELEM_ENTITY; } // ordinal indicator, masculine
&raquo;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (187); return ELEM_ENTITY; } // angle quotation mark, right
&frac14;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (188); return ELEM_ENTITY; } // fraction one-quarter
&frac12;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (189); return ELEM_ENTITY; } // fraction one-half
&frac34;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (190); return ELEM_ENTITY; } // fraction three-quarters
&iquest;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (191); return ELEM_ENTITY; } // inverted question mark
&Agrave;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (192); return ELEM_ENTITY; } // capital A, grave accent
&Aacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (193); return ELEM_ENTITY; } // capital A, acute accent
&Acirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (194); return ELEM_ENTITY; } // capital A, circumflex accent
&Atilde;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (195); return ELEM_ENTITY; } // capital A, tilde
&Auml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (196); return ELEM_ENTITY; } // capital A, dieresis or umlaut
&Aring;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (197); return ELEM_ENTITY; } // capital A, ring
&AElig;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (198); return ELEM_ENTITY; } // capital AE diphthong (ligature)
&Ccedil;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (199); return ELEM_ENTITY; } // capital C, cedilla
&Egrave;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (200); return ELEM_ENTITY; } // capital E, grave accent
&Eacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (201); return ELEM_ENTITY; } // capital E, acute accent
&Ecirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (202); return ELEM_ENTITY; } // capital E, circumflex accent
&Euml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (203); return ELEM_ENTITY; } // capital E, dieresis or umlaut
&Igrave;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (204); return ELEM_ENTITY; } // capital I, grave accent
&Iacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (205); return ELEM_ENTITY; } // capital I, acute accent
&Icirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (206); return ELEM_ENTITY; } // capital I, circumflex accent
&Iuml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (207); return ELEM_ENTITY; } // capital I, dieresis or umlaut
&ETH;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (208); return ELEM_ENTITY; } // capital Eth, Icelandic
&Ntilde;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (209); return ELEM_ENTITY; } // capital N, tilde
&Ograve;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (210); return ELEM_ENTITY; } // capital O, grave accent
&Oacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (211); return ELEM_ENTITY; } // capital O, acute accent
&Ocirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (212); return ELEM_ENTITY; } // capital O, circumflex accent
&Otilde;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (213); return ELEM_ENTITY; } // capital O, tilde
&Ouml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (214); return ELEM_ENTITY; } // capital O, dieresis or umlaut
&times;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (215); return ELEM_ENTITY; } // multiply sign
&Oslash;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (216); return ELEM_ENTITY; } // capital O, slash
&Ugrave;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (217); return ELEM_ENTITY; } // capital U, grave accent
&Uacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (218); return ELEM_ENTITY; } // capital U, acute accent
&Ucirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (219); return ELEM_ENTITY; } // capital U, circumflex accent
&Uuml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (220); return ELEM_ENTITY; } // capital U, dieresis or umlaut
&Yacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (221); return ELEM_ENTITY; } // capital Y, acute accent
&THORN;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (222); return ELEM_ENTITY; } // capital THORN, Icelandic
&szlig;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (223); return ELEM_ENTITY; } // German (sz ligature)
&agrave;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (224); return ELEM_ENTITY; } // small a, grave accent
&aacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (225); return ELEM_ENTITY; } // small a, acute accent
&acirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (226); return ELEM_ENTITY; } // small a, circumflex accent
&atilde;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (227); return ELEM_ENTITY; } // small a, tilde
&auml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (228); return ELEM_ENTITY; } // small a, dieresis or umlaut
&aring;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (229); return ELEM_ENTITY; } // small a, ring
&aelig;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (230); return ELEM_ENTITY; } // small ae diphthong (ligature)
&ccedil;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (231); return ELEM_ENTITY; } // small c, cedilla
&egrave;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (232); return ELEM_ENTITY; } // small e, grave accent
&eacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (233); return ELEM_ENTITY; } // small e, acute accent
&ecirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (234); return ELEM_ENTITY; } // small e, circumflex accent
&euml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (235); return ELEM_ENTITY; } // small e, dieresis or umlaut
&igrave;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (236); return ELEM_ENTITY; } // small i, grave accent
&iacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (237); return ELEM_ENTITY; } // small i, acute accent
&icirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (238); return ELEM_ENTITY; } // small i, circumflex accent
&iuml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (239); return ELEM_ENTITY; } // small i, dieresis or umlaut
&eth;?          {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (240); return ELEM_ENTITY; } // small eth, Icelandic
&ntilde;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (241); return ELEM_ENTITY; } // small n, tilde
&ograve;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (242); return ELEM_ENTITY; } // small o, grave accent
&oacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (243); return ELEM_ENTITY; } // small o, acute accent
&ocirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (244); return ELEM_ENTITY; } // small o, circumflex accent
&otilde;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (245); return ELEM_ENTITY; } // small o, tilde
&ouml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (246); return ELEM_ENTITY; } // small o, dieresis or umlaut
&divide;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (247); return ELEM_ENTITY; } // divide sign
&oslash;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (248); return ELEM_ENTITY; } // small o, slash
&ugrave;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (249); return ELEM_ENTITY; } // small u, grave accent
&uacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (250); return ELEM_ENTITY; } // small u, acute accent
&ucirc;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (251); return ELEM_ENTITY; } // small u, circumflex accent
&uuml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (252); return ELEM_ENTITY; } // small u, dieresis or umlaut
&yacute;?       {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (253); return ELEM_ENTITY; } // small y, acute accent
&thorn;?        {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (254); return ELEM_ENTITY; } // small thorn, Icelandic
&yuml;?         {if (_HANDLE_ENTITY)   _HANDLE_ENTITY      (255); return ELEM_ENTITY; } // small y, dieresis or umlaut

       /* tokenization */
{TOKEN}+	{if (_HANDLE_TOKEN)    _HANDLE_TOKEN    (ELEM_TOKEN, yytext, yyleng); return ELEM_TOKEN; }


[^ \n\r\t]      {if (_HANDLE_PUNCT)    _HANDLE_PUNCT    (ELEM_PUNCT, yytext, yyleng); return ELEM_PUNCT; }
.|\n|\r\|t	{if (_HANDLE_UNMATCHED)_HANDLE_UNMATCHED(ELEM_UNMATCH, yytext, yyleng); return ELEM_UNMATCH; }

<<EOF>> 	{ return ELEM_EOF; }
%%








/*----------- functions -----------*/

/******************************
 * function  : echoOpenTag()
 * parameters: the tag, its length
 * returns   : error code, 0 if everything ok
 *
 * Called when an open tag is found and prints it out
******************************/
int echoOpenTag(int id, char* tag, int len)
{
  /* debug */ printf("OPEN TAG: %s\n", tag);
  return 0; 	/* everything ok */
}





/******************************
 * function  : echoCloseTag()
 * parameters: the tag, its length
 * returns   : error code, 0 if everything ok
 *
 * Called when a close tag is found and prints it out
******************************/
int echoCloseTag(int id, char* tag, int len)
{
  /* debug */ printf("CLOSE TAG: %s\n", tag);
  return 0; 	/* everything ok */
}





/******************************
 * function  : echoToken()
 * parameters: the token, its length
 * returns   : error code, 0 if everything ok
 *
 * Called when a token is found and prints it out
******************************/
int echoToken(int id, char* token, int len)
{
  /* debug */ printf("TOKEN %s\n", token);
  return 0; 	/* everything ok */
}





/******************************
 * function  : handleComment()
 * parameters: the comment, its length
 * returns   : error code, 0 if everything ok
 *
 * Called when comments are found, i.e. <! ... > and prints it out
******************************/
int echoComment(int id, char* comment, int len)
{
  /* debug */ printf("COMMENT %s\n", comment);
  return 0;
}





/******************************
 * function  : echoPunct()
 * parameters: the punctuation mark, its length
 * returns   : error code, 0 if everything ok
 *
 * Called when a punct mark is found. A punct is essentially
 * any character that is not matched as a token. It prints the token out.
******************************/
int echoPunct(int id, char* punct, int len)
{
  /* debug */ printf("PUNCT %s\n", punct);
  return 0;
}





/******************************
 * function  : echoUnmatched()
 * parameters: a character that was not matched by
 *             any of the rules, its length
 * returns   : error code, 0 if everything ok
 * 
 * Called when we came up to a character that
 * was not matched and prints it out
******************************/
int echoUnmatched(int id, char* str, int len)
{
  /* debug */ printf("UNMATCHED chr(%d)\n", *str);
  return 0;
}  





/******************************
 * function : echoEntity()
 * paramters: the entity
 * reutrns  : error code, 0 if everything ok
 *
 * Called when there is an entity. The entity is not
 * passed as string, but a decimal value is passed instead. It prints
 * this value out.
******************************/
int echoEntity(int ent)
{
  /* debug */ printf("ENTITY %c\n", ent);
}






/******************************
 * function  : useString()
 * parameters: the string we would like to use
 *
 * makes the parser use the specified string
******************************/
void useString(char *str)
{ yy_scan_string(str); }



/******************************
 * function  : useBuffer()
 * parameters: the buffer we would like to use
 *
 * makes the parser use the specified buffer
******************************/
void useBuffer(char *str, int len)
{ yy_scan_buffer(str, len); }



/******************************
 * function  : deleteCurrentBuf()
 * 
 * deletes the current buffer
******************************/
void deleteCurrentBuf()
{ yy_flush_buffer( YY_CURRENT_BUFFER ); yy_delete_buffer( YY_CURRENT_BUFFER ); }



/******************************
 * function  : yywrap()
 *
 * default implementation
 *****************************/
int yywrap() 
{ return 1; }


/******************************
 * functions  : setXHandler
 * set the function to use as handlers
******************************/
void setOpenTagHandler   (ACTION_PTR h)     { _HANDLE_OPEN_TAG  = h; }
void setCloseTagHandler  (ACTION_PTR h)     { _HANDLE_CLOSE_TAG = h; }
void setTokenHandler     (ACTION_PTR h)     { _HANDLE_TOKEN     = h; }
void setCommentHandler   (ACTION_PTR h)     { _HANDLE_COMMENT   = h; }
void setPunctHandler     (ACTION_PTR h)     { _HANDLE_PUNCT     = h; }
void setUnmatchedHandler (ACTION_PTR h)     { _HANDLE_UNMATCHED = h; }
void setEntityHandler    (ACTION_PTR_ENT h) { _HANDLE_ENTITY    = h; }


/******************************
 * function  : backtrack()
 * parameters: how much to backtrack
 *
 * calls yyless()
 *****************************/
void backtrack(int l)
{ yyless(l); }


/******************************
 * function  : printTags
 * parameters: a null terminated string to analyze
 *
 * Prints out the opening and closing of tags. Can be used
 * for testing
******************************/
void printTags(char* str)
{
  char tags[100][100] = { "_UKN_", "A","ABBR","ACRONYM","ADDRESS",
                          "APPLET","AREA","B","BASE","BASEFONT","BDO","BIG",
			  "BLOCKQUOTE","BODY","BR","BUTTON","CAPTION","CENTER",
			  "CITE","CODE","COL","COLGROUP","DD","DEL","DFN","DIR",
			  "DIV","DL","DT","EM","FIELDSET","FONT","FORM","FRAME",
			  "FRAMESET","H1","H2","H3","H4","H5","H6","HEAD","HR","HTML",
			  "I","IFRAME","IMG","INPUT","INS","ISINDEX","KBD","LABEL",
			  "LEGEND","LI","LINK","MAP","MENU","META","NOFRAMES","NOSCRIPT",
			  "OBJECT","OL","OPTGROUP","OPTION","P","PARAM","PRE","Q","S",
			  "SAMP","SCRIPT","SELECT","SMALL","SPAN","STRIKE","STRONG",
			  "STYLE","SUB","SUP","TABLE","TBODY","TD","TEXTAREA","TFOOT",
			  "TH","THEAD","TITLE","TR","TT","U","UL","VAR"};
  YY_BUFFER_STATE bufstate = yy_scan_string(str);

  int i = ELEM_UNKNOWN ;
  while ((i = yylex())  != ELEM_EOF)
    if (i < ELEM_TOKEN)
    {
      if (i < ELEM_CLOSE) printf("OPEN  %s %d\n", tags[i], i); 
      else         printf("CLOSE /%s %d\n", tags[i-ELEM_CLOSE], i);
    }
    else if (i == ELEM_TOKEN) printf("TOKEN %s\n", yytext);
  yy_delete_buffer(YY_CURRENT_BUFFER);
}



#ifdef _STANDALONE
/************
 * test main
************/
int main(int argc, char **argv) 
{
  char tmp[1024 * 1024];

  _HANDLE_UNMATCHED = NULL;

  FILE *fin;

  if (argc > 1)
    fin = fopen(argv[1], "r"); 
  else 
    fin = stdin;

  fread(tmp, sizeof(char), 1024 * 1024, fin);
/*
_HANDLE_OPEN_TAG = NULL;
_HANDLE_CLOSE_TAG = NULL;
_HANDLE_TOKEN = NULL;
_HANDLE_COMMENT = NULL;
_HANDLE_PUNCT = NULL;
_HANDLE_UNMATCHED = NULL;
_HANDLE_ENTITY = NULL;

  printTags(tmp);
  return;
  */

  YY_BUFFER_STATE bufstate = yy_scan_string(tmp);

  while (yylex() != ELEM_EOF);

  yy_delete_buffer(YY_CURRENT_BUFFER);
  return 0;
}
#endif

