//
// ricExtractorInterface.h
//
// @author Ka Cheung Sia {kcsia@cs.ucla.edu}
// @date   02/01/2005
// @copyright Ka Cheung Sia
//
// Implementation of the defined interface for developer to add more regular expression
//
//
// Copyright (c) 2005, Ka Cheung Sia {kcsia@cs.ucla.edu}
// University of California, Los Angeles
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of the University of California Los Angeles nor
// the names of its contributors may be used to endorse or promote
// products derived from this software without specific prior written
// permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "ricExtractorInterface.h"

using namespace kcsia;

int ricDateExtractorInterface::convertYear (boost::match_results<std::string::const_iterator> results, int position) {
    int year=atoi(results[position].str().c_str());
    if (year < 50)
        year=year+2000;
    else if (year<100 && year >=50)
        year=year+1900;
    return year;
}

int ricDateExtractorInterface::convertMonthNumeric (boost::match_results<std::string::const_iterator> results, int position) {
    return atoi(results[position].str().c_str());
}

int ricDateExtractorInterface::convertMonthString (boost::match_results<std::string::const_iterator> results, int position) {
    int month;
    const char* c_month=results[position].str().c_str();
    if (strstr(c_month,"Jan")!=NULL || strstr(c_month,"jan")!=NULL || strstr(c_month,"JAN")!=NULL )
        month=1;
    else if (strstr(c_month,"Feb")!=NULL || strstr(c_month,"feb")!=NULL || strstr(c_month,"FEB")!=NULL)
        month=2;
    else if (strstr(c_month,"Mar")!=NULL || strstr(c_month,"mar")!=NULL || strstr(c_month,"MAR")!=NULL)
        month=3;
    else if (strstr(c_month,"Apr")!=NULL || strstr(c_month,"apr")!=NULL || strstr(c_month,"APR")!=NULL)
        month=4;
    else if (strstr(c_month,"May")!=NULL || strstr(c_month,"may")!=NULL || strstr(c_month,"MAY")!=NULL)
        month=5;
    else if (strstr(c_month,"Jun")!=NULL || strstr(c_month,"jun")!=NULL || strstr(c_month,"JUN")!=NULL)
        month=6;
    else if (strstr(c_month,"Jul")!=NULL || strstr(c_month,"jul")!=NULL || strstr(c_month,"JUL")!=NULL)
        month=7;
    else if (strstr(c_month,"Aug")!=NULL || strstr(c_month,"aug")!=NULL || strstr(c_month,"AUG")!=NULL)
        month=8;
    else if (strstr(c_month,"Sep")!=NULL || strstr(c_month,"sep")!=NULL || strstr(c_month,"SEP")!=NULL)
        month=9;
    else if (strstr(c_month,"Oct")!=NULL || strstr(c_month,"oct")!=NULL || strstr(c_month,"OCT")!=NULL)
        month=10;
    else if (strstr(c_month,"Nov")!=NULL || strstr(c_month,"nov")!=NULL || strstr(c_month,"NOV")!=NULL)
        month=11;
    else if (strstr(c_month,"Dec")!=NULL || strstr(c_month,"dec")!=NULL || strstr(c_month,"DEC")!=NULL)
        month=12;
    else 
        month=0;
    return month;
}

int ricDateExtractorInterface::convertDay (boost::match_results<std::string::const_iterator> results, int position) {
    return atoi(results[position].str().c_str());
}


int ricDateExtractorImpl::getYear (boost::match_results<std::string::const_iterator> results) {
    switch (regex_id) {
    case 0:
    case 1:
    case 2:
    case 5:
    case 7:
        return convertYear(results,4);
    case 4:
    case 6:
    case 9:
    case 10:
    case 11:
    case 12:
    case 13:
        return convertYear(results,3);
    case 8:
        return convertYear(results,1);
    default:
        return 1994;
    }
}

int ricDateExtractorImpl::getMonth (boost::match_results<std::string::const_iterator> results) {
    switch (regex_id) {
    case 5:
    case 8:
        return convertMonthNumeric(results,2);
    case 9:
    case 10:
    case 11:
        return convertMonthNumeric(results,1);
    case 4:
    case 6:
    case 7:
        return convertMonthString(results,1);
    case 0:
    case 3:
    case 12:
    case 13:
        return convertMonthString(results,2);
    case 1:
    case 2:
        return convertMonthString(results,3);
    default:
        return 0;
    }
}

int ricDateExtractorImpl::getDay (boost::match_results<std::string::const_iterator> results) {
    switch (regex_id) {
    case 0:
    case 5:
    case 8:
    case 3:
        return convertDay(results,3);
    case 1:
    case 12:
    case 13:
        return convertDay(results,1);
    case 2:
    case 4:
    case 6:
    case 7:
    case 9:
    case 10:
    case 11:
        return convertDay(results,2);
    default:
        return 0;
    }
}

int ricTimeExtractorInterface::convertHour12 (boost::match_results<std::string::const_iterator> results, int position, int mposition) {
    int hour=atoi(results[position].str().c_str());
    const char *apm=results[mposition].str().c_str();
    if (apm[0]=='a' || apm[0]=='A') {
        if (hour == 12)
            hour=0;
    } else {
        if (hour < 12)
            hour+=12;
    }
    return hour;
}


int ricTimeExtractorInterface::convertHour24 (boost::match_results<std::string::const_iterator> results, int position) {
    return atoi(results[position].str().c_str());
}

int ricTimeExtractorInterface::convertMin (boost::match_results<std::string::const_iterator> results, int position) {
    return atoi(results[position].str().c_str());
}


int ricTimeExtractorImpl::getHour (boost::match_results<std::string::const_iterator> results) {
    if (regex_id==0 || regex_id==1 || ((regex_id==2) && results.size()==3)) {
        return convertHour12(results,1,3);
    } else {
        return convertHour24(results,1);
    }
}

int ricTimeExtractorImpl::getMin (boost::match_results<std::string::const_iterator> results) {
    return convertMin(results,2);
}
