//
// ricExtractorInterface.h
//
// @author Ka Cheung Sia {kcsia@cs.ucla.edu}
// @date   02/01/2005
// @copyright Ka Cheung Sia
//
// Defintion of an interface for developer to add mroe regular expression
// A Simple implementation for the defined interface
//
//
// Copyright (c) 2005, Ka Cheung Sia {kcsia@cs.ucla.edu}
// University of California, Los Angeles
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of the University of California Los Angeles nor
// the names of its contributors may be used to endorse or promote
// products derived from this software without specific prior written
// permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
// TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include <boost/regex.hpp>
#include <string>

#ifndef __RIC_EXTRACTORINTERFACE_H
#define __RIC_EXTRACTORINTERFACE_H

namespace kcsia {

    // this act as a interface (abstract class)
    class ricDateExtractorInterface {
    protected:
        // helper function of extracted date from regex result
        int convertYear (boost::match_results<std::string::const_iterator> results, int position);
        int convertMonthNumeric (boost::match_results<std::string::const_iterator> results, int position);
        int convertMonthString (boost::match_results<std::string::const_iterator> results, int position);
        int convertDay (boost::match_results<std::string::const_iterator> results, int position);


    public:
        // interfaces that a sub-class must implement
        virtual int getYear (boost::match_results<std::string::const_iterator> results)=0;
        virtual int getMonth (boost::match_results<std::string::const_iterator> results)=0;
        virtual int getDay (boost::match_results<std::string::const_iterator> results)=0;
        
    };


    class ricTimeExtractorInterface {

    protected:
        // helper function of extracting time from regex result
        int convertHour12 (boost::match_results<std::string::const_iterator> results, int position, int mposition);

        int convertHour24 (boost::match_results<std::string::const_iterator> results, int position);

        int convertMin (boost::match_results<std::string::const_iterator> results, int position);

    public:
        // interfaces that a subclass must implement
        virtual int getHour (boost::match_results<std::string::const_iterator> results)=0;
        virtual int getMin (boost::match_results<std::string::const_iterator> results)=0;
    };


    class ricDateExtractorImpl : public ricDateExtractorInterface {
    private:
        int regex_id;
    public:
        ricDateExtractorImpl(int id) : regex_id(id) {}
        // interfaces that a sub-class must implement
        virtual int getYear (boost::match_results<std::string::const_iterator> results);
        virtual int getMonth (boost::match_results<std::string::const_iterator> results);
        virtual int getDay (boost::match_results<std::string::const_iterator> results);
    };

    class ricTimeExtractorImpl : public ricTimeExtractorInterface {
    private:
        int regex_id;
    public:
        ricTimeExtractorImpl(int id) : regex_id(id) {}
        // interfaces that a subclass must implement
        virtual int getHour (boost::match_results<std::string::const_iterator> results);
        virtual int getMin (boost::match_results<std::string::const_iterator> results);
    };

}


#endif
